#%% Images are NumPy’s arrays np.ndarray
import skimage 
import numpy as np
import os 
import matplotlib
import matplotlib.pyplot as plt
import scipy #this is required for the 2d convolution against an arbirary filter
import math 
from skimage import data  # most functions are in subpackages
from skimage import io #Input/output, data types and colorspaces
from scipy import signal # this is required for the 2d convolution

# this sets the image size
matplotlib.rcParams["figure.figsize"]=(15,12)

#load you favourite image
cameraman256 = data.camera()
# by default images are uint8
print('cameraman256.dtype = ' + str(cameraman256.dtype))

print('cameraman256.shape = ' + str(cameraman256.shape))


# image visualization
plt.imshow(cameraman256, cmap='gray')
plt.show()


#%% float images are supposed to lie in [-1, 1] (in order to have comparable contrast for all float images)

# we can convert images
camera_float = skimage.img_as_float(cameraman256)

print('cameraman256.max() ' + str(cameraman256.max()))
print('cameraman256.min() ' + str(cameraman256.min()))
print('camera_float.max() ' + str(camera_float.max()))
print('camera_float.min() ' + str(camera_float.min()))

plt.imshow(camera_float, cmap='gray'),
plt.title('cameraman')
plt.show()

#%% Color images
#filename = os.path.join(skimage.data_dir, 'camera.png')
img = data.chelsea()
# by default images are uint8
print('chelsea.dtype = ' + str(img.dtype))
print('chelsea.shape = ' + str(img.shape))

# image visualization
plt.imshow(img)
plt.title('chelsea')
plt.show()

#%% image Filtering (convolution)
SZ = 35
h = np.ones((SZ, SZ));
h = h / np.sum(h);

smoothedCameramann = signal.convolve2d(cameraman256, h, boundary='fill', mode='same')
plt.imshow(smoothedCameramann, cmap = 'gray')
plt.title('cameraman smoothed')
plt.show()

# derivative filters, computer gradient magnitude
hy = np.array([[1, 1, 1], [0, 0, 0], [-1, -1, -1]]);
hx = hy.T

dx = signal.convolve2d(camera_float, hx, boundary='fill', mode='same')
dy = signal.convolve2d(camera_float, hy, boundary='fill', mode='same')

plt.imshow(np.hstack((dx, dy)), vmin=-1, vmax=1, cmap = 'coolwarm')
plt.title('cameraman [horz, vert] derivatives')
plt.colorbar()
plt.show()

# computing gradient magnitude
gradMagn = np.sqrt(dx **2 + dy **2)
# computing gradient direction
gradDir = np.arctan2(dy, dx)

plt.imshow(gradMagn, cmap = 'gray')
plt.title('cameraman gradient magnitude')
plt.colorbar()
plt.show()

plt.imshow(gradDir, cmap = 'viridis')
plt.title('cameraman gradient direction')
plt.colorbar()
plt.show()

# create a mask for the gradient to remove pixels where the gredient magnitude is low
gradMask = np.zeros_like(camera_float)
gradMask[gradMagn > 0.5] = 1
plt.imshow(gradMask, cmap = 'gray')
plt.title('Gradient Magnitude Mask')

gradDirMasked = gradDir.copy()
gradDirMasked[gradMask==0] = np.nan   #set to nan to make it black in the figure
plt.imshow(gradDirMasked, vmin=-np.pi, vmax=np.pi, cmap = 'viridis')
plt.colorbar()
plt.show()